@@ -38,7 +38,7 @@ func TestWriteGitSSLConfig(t *testing.T) {
 		},
 	}
 
-	mockWriter := new(MockShellWriter)
+	mockWriter := NewMockShellWriter(t)
 	mockWriter.On("EnvVariableKey", tls.VariableCAFile).Return("VariableCAFile").Once()
 	mockWriter.On("EnvVariableKey", tls.VariableCertFile).Return("VariableCertFile").Once()
 	mockWriter.On("EnvVariableKey", tls.VariableKeyFile).Return("VariableKeyFile").Once()
@@ -66,8 +66,6 @@ func TestWriteGitSSLConfig(t *testing.T) {
 	).Once()
 
 	shell.writeGitSSLConfig(mockWriter, build, nil)
-
-	mockWriter.AssertExpectations(t)
 }
 
 func getJobResponseWithMultipleArtifacts() common.JobResponse {
@@ -123,8 +121,7 @@ func TestWriteWritingArtifactsOnSuccess(t *testing.T) {
 		Build:         build,
 	}
 
-	mockWriter := new(MockShellWriter)
-	defer mockWriter.AssertExpectations(t)
+	mockWriter := NewMockShellWriter(t)
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("TmpFile", "gitlab_runner_env").Return("path/to/env/file").Once()
 	mockWriter.On("SourceEnv", "path/to/env/file").Once()
@@ -195,8 +192,7 @@ func TestWriteWritingArtifactsOnFailure(t *testing.T) {
 		Build:         build,
 	}
 
-	mockWriter := new(MockShellWriter)
-	defer mockWriter.AssertExpectations(t)
+	mockWriter := NewMockShellWriter(t)
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("TmpFile", "gitlab_runner_env").Return("path/to/env/file").Once()
 	mockWriter.On("SourceEnv", "path/to/env/file").Once()
@@ -272,8 +268,7 @@ func TestWriteWritingArtifactsWithExcludedPaths(t *testing.T) {
 		Build:         build,
 	}
 
-	mockWriter := new(MockShellWriter)
-	defer mockWriter.AssertExpectations(t)
+	mockWriter := NewMockShellWriter(t)
 	mockWriter.On("Variable", mock.Anything)
 	mockWriter.On("TmpFile", "gitlab_runner_env").Return("path/to/env/file").Once()
 	mockWriter.On("SourceEnv", "path/to/env/file").Once()
@@ -374,8 +369,7 @@ func TestWriteWritingArchiveCacheOnSuccess(t *testing.T) {
 				Build:         build,
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("Variable", mock.MatchedBy(func(v common.JobVariable) bool {
 				return v.Key == "GITLAB_ENV"
 			})).Once()
@@ -501,8 +495,7 @@ func TestWriteWritingArchiveCacheOnFailure(t *testing.T) {
 				Build:         build,
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("Variable", mock.MatchedBy(func(v common.JobVariable) bool {
 				return v.Key == "GITLAB_ENV"
 			})).Once()
@@ -724,6 +717,7 @@ func TestAbstractShell_handleGetSourcesStrategy(t *testing.T) {
 							Variables: common.JobVariables{
 								{Key: "GIT_STRATEGY", Value: tc.gitStrategy},
 							},
+							JobRequestCorrelationID: "foobar",
 						},
 						BuildDir: tc.buildDir,
 					}
@@ -874,9 +868,7 @@ func TestAbstractShell_writeCleanupBuildDirectoryScript(t *testing.T) {
 					BuildDir: tc.buildDir,
 				},
 			}
-			mockShellWriter := &MockShellWriter{}
-			defer mockShellWriter.AssertExpectations(t)
-
+			mockShellWriter := NewMockShellWriter(t)
 			tc.setupExpectations(mockShellWriter)
 			shell := AbstractShell{}
 
@@ -930,9 +922,7 @@ func TestGitCleanFlags(t *testing.T) {
 				},
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
-
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("Noticef", "Checking out %s as detached HEAD (ref is %s)...", dummySha[0:8], dummyRef).Once()
 			mockWriter.On("Command", "git", "-c", "submodule.recurse=false", "checkout", "-f", "-q", dummySha).Once()
 
@@ -1317,9 +1307,7 @@ func TestAbstractShell_extractCacheWithDefaultFallbackKey(t *testing.T) {
 				Build:         build,
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
-
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("IfCmd", "runner-command", "--version").Once()
 			mockWriter.On("Noticef", "Checking cache for %s...", testCacheKey).Once()
 
@@ -1534,9 +1522,7 @@ func TestAbstractShell_extractCacheWithMultipleFallbackKeys(t *testing.T) {
 				Build:         build,
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
-
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("IfCmd", "runner-command", "--version").Once()
 
 			for _, cacheKey := range tc.allowedCacheKeys {
@@ -1700,9 +1686,7 @@ func TestAbstractShell_extractCacheWithMultipleFallbackKeysWithCleanup(t *testin
 				Build:         build,
 			}
 
-			mockWriter := new(MockShellWriter)
-			defer mockWriter.AssertExpectations(t)
-
+			mockWriter := NewMockShellWriter(t)
 			mockWriter.On("IfCmd", "runner-command", "--version").Once()
 
 			for _, cacheKey := range tc.allowedCacheKeys {
@@ -2067,9 +2051,7 @@ func TestWriteUserScript(t *testing.T) {
 				},
 				PostBuildScript: tt.postBuildScript,
 			}
-			mockShellWriter := &MockShellWriter{}
-			defer mockShellWriter.AssertExpectations(t)
-
+			mockShellWriter := NewMockShellWriter(t)
 			tt.setupExpectations(mockShellWriter)
 			shell := AbstractShell{}
 
@@ -2263,8 +2245,7 @@ func TestScriptSections(t *testing.T) {
 					},
 					PostBuildScript: "echo postbuild",
 				}
-				mockShellWriter := &MockShellWriter{}
-				defer mockShellWriter.AssertExpectations(t)
+				mockShellWriter := NewMockShellWriter(t)
 
 				tt.setupExpectations(mockShellWriter)
 				shell := AbstractShell{}
@@ -2570,7 +2551,6 @@ func TestAbstractShell_writeCleanupScript(t *testing.T) {
 					}
 
 					mockShellWriter := NewMockShellWriter(t)
-
 					mockShellWriter.On("TmpFile", "masking.db").Return("masking.db").Once()
 					mockShellWriter.On("RmFile", "masking.db").Once()
 
@@ -2707,14 +2687,13 @@ func TestWriteUploadArtifactIncludesGenerateArtifactsMetadataArgs(t *testing.T)
 				args = append(args, "--artifact-format", string(f))
 			}
 
-			shellWriter := &MockShellWriter{}
+			shellWriter := NewMockShellWriter(t)
 			shellWriter.On("IfCmd", mock.Anything, mock.Anything).Once()
 			shellWriter.On("Noticef", "Uploading artifacts...").Once()
 			shellWriter.On("Command", args...).Once()
 			shellWriter.On("Else").Once()
 			shellWriter.On("Warningf", mock.Anything, mock.Anything, mock.Anything).Once()
 			shellWriter.On("EndIf").Once()
-			defer shellWriter.AssertExpectations(t)
 
 			shell := &AbstractShell{}
 			shell.writeUploadArtifact(shellWriter, info, common.Artifact{
@@ -2889,6 +2868,7 @@ func TestAbstractShell_writeGetSourcesScript_scriptHooks(t *testing.T) {
 										Script: common.StepScript{"job payload", "post_get_sources"},
 									},
 								},
+								JobRequestCorrelationID: "foobar",
 							},
 							Runner: &common.RunnerConfig{
 								RunnerSettings: common.RunnerSettings{
@@ -2988,6 +2968,8 @@ func expectSetupTemplate(shellWriter *MockShellWriter, dir string) (string, []*m
 		shellWriter.EXPECT().Command("git", "config", "-f", config, "credential.interactive", "never").Once(),
 		shellWriter.EXPECT().Command("git", "config", "-f", mock.Anything, "gc.autoDetach", "false").Once(),
 		shellWriter.EXPECT().Command("git", "config", "-f", config, "transfer.bundleURI", "true").Maybe(),
+		shellWriter.EXPECT().Command("git", "config", "-f", config, "http.extraHeader", "X-Gitaly-Correlation-ID: foobar").Maybe(),
+		shellWriter.EXPECT().Noticef("Gitaly correlation ID: %s", "foobar").Maybe(),
 	}
 	mock.InOrder(calls...)
 
@@ -3111,6 +3093,7 @@ func TestAbstractShell_writeGitCleanup(t *testing.T) {
 										GitInfo: common.GitInfo{
 											RepoURL: "https://repo-url/some/repo",
 										},
+										JobRequestCorrelationID: "foobar",
 									},
 									Runner: &common.RunnerConfig{
 										RunnerSettings: common.RunnerSettings{
